@@ -18,9 +18,6 @@ my $nick = "spleen" . ($$ % 1000);
 sub _start {
   my ($kernel, $session) = @_[KERNEL, SESSION];
 
-  # Make an alias for our session, to keep it from getting GC'ed.
-  $kernel->alias_set( 'smileyninja' );
-
   # Ask the IRC component to send us all IRC events it receives. This
   # is the easy, indiscriminate way to do it.
   $kernel->post( 'test', 'register', 'all');
@@ -29,8 +26,7 @@ sub _start {
   # sent to and received from the IRC server. Very useful for debugging.
   $kernel->post( 'test', 'connect', { Debug    => 1,
 				      Nick     => $nick,
-                                      Server   => $ARGV[0] ||
-				                  'binky.phreeow.net',
+                                      Server   => $ARGV[0] || 'irc.phreeow.net',
 				      Port     => $ARGV[1] || 6667,
 				      Username => 'neenio',
 				      Ircname  => 'Ask me about my colon!', }
@@ -72,8 +68,7 @@ sub _stop {
   my ($kernel) = $_[KERNEL];
 
   print "Control session stopped.\n";
-  $kernel->post( 'test', 'quit', 'Neenios on ice!' );
-  $kernel->alias_remove( 'smileyninja' );
+  $kernel->call( 'test', 'quit', 'Neenios on ice!' );
 }
 
 
